﻿Type=Class
Version=4.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
Sub Class_Globals
	Private sql1 As SQL
	Private ser As B4XSerializator
	Private UNIVERSE_SETTINGS = "Universe_Settings", TIME_MODIFIED = "Time Modified" As String
	Private Now = DateTime.Date(DateTime.Now) & " " & DateTime.Time(DateTime.Now) As String
End Sub

'Initializes the Universe and sets the Universe file.
Public Sub Initialize (Dir As String, FileName As String)
	If FileName.Contains(".universe") = False Then
		FileName = FileName & ".universe"
	End If
	If sql1.IsInitialized Then sql1.Close
	#If B4J
	sql1.InitializeSQLite(Dir, FileName, True)
	#Else
	sql1.Initialize(Dir, FileName, True)
	#End If
	CreateTable("Universe_Settings")
End Sub

#Region "Table Actions"
'Creates the provided table if it doesn't exist.
Public Sub CreateTable (Table As String)
	Table = CorrectTable(Table)
	sql1.ExecNonQuery("CREATE TABLE IF NOT EXISTS " & Table & "(key TEXT PRIMARY KEY, value NONE)")
	InsertTimeModified
End Sub

'Drops (Deletes) the provided table.
Public Sub DropTable(Table As String)
	Table = CorrectTable(Table)
	sql1.ExecNonQuery($"DROP TABLE '${Table}'"$)
	
End Sub

Public Sub RenameTable(OldTable As String, NewTable As String)
	OldTable = CorrectTable(OldTable)
	NewTable = CorrectTable(NewTable)
	sql1.ExecNonQuery("ALTER TABLE " & OldTable & " RENAME TO " & NewTable)
End Sub

'Tests whether a table is available in the store.
Public Sub ContainsTable(Table As String) As Boolean
	Table = CorrectTable(Table)
	Dim Answer As Int = sql1.ExecQuerySingleResult($"SELECT count(*) FROM sqlite_master WHERE type = 'table' AND name = '${Table}'"$)
	If Answer = 1 Then
		Return True
	Else
		Return False
	End If
End Sub

'Returns a list of tables.
Public Sub ListTables As List
	Dim c As ResultSet = sql1.ExecQuery("SELECT name FROM sqlite_master WHERE type='table'")
	Dim res As List
	res.Initialize
	Do While c.NextRow
		res.Add(c.GetString2(0))
	Loop
	c.Close
	Return res
End Sub

'Drops and Re-Creates the table. If the table doesn't exist, this only Creates the table.
Public Sub ReloadTable (Table As String)
	Table = CorrectTable(Table)
	If ContainsTable(Table) = True Then
		DropTable(Table)
		CreateTable(Table)
	Else
		CreateTable(Table)
	End If
End Sub
#End Region

#Region "Key Actions"
'Returns a list with all the keys.
Public Sub ListKeys (Table As String) As List
	Table = CorrectTable(Table)
	Dim c As ResultSet = sql1.ExecQuery("SELECT key FROM " & Table)
	Dim res As List
	res.Initialize
	Do While c.NextRow
		res.Add(c.GetString2(0))
	Loop
	c.Close
	Return res
End Sub

'Tests whether a key is available in the store.
Public Sub ContainsKey(Table As String, Key As String) As Boolean
	Table = CorrectTable(Table)
	Return sql1.ExecQuerySingleResult2("SELECT count(key) FROM " & Table & " WHERE key = ?", Array As String(Key)) > 0
End Sub
'Removes the key and value mapped to this key.
Public Sub RemoveKey(Table As String, Key As String)
	Table = CorrectTable(Table)
	sql1.ExecNonQuery2("DELETE FROM " & Table & " WHERE key = ?", Array As Object(Key))
	
End Sub
#End Region

#Region "Main Actions"
Public Sub Put(Table As String, Key As String, Value As Object)
	Table = CorrectTable(Table)
	sql1.ExecNonQuery2("INSERT OR REPLACE INTO " & Table & " VALUES(?, ?)", Array As Object(Key, ser.ConvertObjectToBytes(Value)))
	
End Sub

Public Sub PutEncrypted (Table As String, Key As String, Value As Object, Password As String)
	Table = CorrectTable(Table)
	#If B4I
		Dim C As Cipher
	#Else
		Dim C As B4XCipher
	#End If
	Put(Table, Key, C.Encrypt(ser.ConvertObjectToBytes(Value), Password))
End Sub

#If B4J
Public Sub PutImage (Table As String, Key As String, Value As Image)
	Dim Output As OutputStream
	Output.InitializeToBytesArray(0)
	Value.WriteToStream(Output)
	Put(Table, Key, Output.ToBytesArray)
	Output.Close
End Sub

Public Sub PutImageEncrypted (Table As String, Key As String, Value As Image, Password As String)
	Dim Output As OutputStream
	Output.InitializeToBytesArray(0)
	Value.WriteToStream(Output)
	PutEncrypted(Table, Key, Value, Password)
End Sub
#Else
Public Sub PutBitmap(Table As String, Key As String, Value As Bitmap)
	Dim out As OutputStream
	out.InitializeToBytesArray(0)
	Value.WriteToStream(out, 100, "PNG")
	Put(Key, out.ToBytesArray)
	out.Close
End Sub

Public Sub PutBitmapEncrypted(Table As String, Key As String, Value As Bitmap, Password As String)
	Dim out As OutputStream
	out.InitializeToBytesArray(0)
	Value.WriteToStream(out, 100, "PNG")
	Put(Table, Key, out.ToBytesArray)
	out.Close
End Sub
#End If

Public Sub Get(Table As String, Key As String) As Object
	Table = CorrectTable(Table)
	Dim rs As ResultSet = sql1.ExecQuery2("SELECT value FROM " & Table & " WHERE key = ?", Array As String(Key))
	Dim result As Object = Null
	If rs.NextRow Then
		result = ser.ConvertBytesToObject(rs.GetBlob2(0))
	End If
	rs.Close
	Return result
End Sub

Public Sub GetDefault(Table As String, Key As String, DefaultValue As Object) As Object
	Table = CorrectTable(Table)
	Dim res As Object = Get(Key, Table)
	If res = Null Then Return DefaultValue
	Return res
End Sub

Public Sub GetEncrypted (Table As String, Key As String, Password As String) As Object
	Table = CorrectTable(Table)
	#If B4I
		Dim C As Cipher
	#Else
		Dim C As B4XCipher
	#End If
	Dim b() As Byte = Get(Table, Key)
	If b = Null Then Return Null
	Return ser.ConvertBytesToObject(C.Decrypt(b, Password))
End Sub

#If B4J
Public Sub GetImage (Table As String, Key As String) As Image
	Dim B() As Byte = Get(Table, Key)
	If B = Null Then Return Null
	Dim Input As InputStream
	Input.InitializeFromBytesArray(B, 0, B.Length)
	Dim Image As Image
	Image.Initialize2(Input)
	Input.Close
	Return Image
End Sub

Public Sub GetImageEncrypted (Table As String, Key As String, Password As String) As Image
	Dim B() As Byte = GetEncrypted(Table, Key, Password)
	If B = Null Then Return Null
	Dim Input As InputStream
	Input.InitializeFromBytesArray(B, 0, B.Length)
	Dim Image As Image
	Image.Initialize2(Input)
	Input.Close
	Return Image	
End Sub
#Else
Public Sub GetBitmap(Table As String, Key As String) As Bitmap
	Dim B() As Byte = Get(Table, Key)
	If B = Null Then Return Null
	Dim Input As InputStream
	Input.InitializeFromBytesArray (B, 0, B.Length)
	Dim Bitmap As Bitmap
	Bitmap.Initialize2(Input)
	Input.Close
	Return Bitmap
End Sub

Public Sub GetBitmapEncrypted(Table As String, Key As String, Password As String) As Bitmap
	Dim B() As Byte = GetEncrypted(Table, Key, Password)
	If B = Null Then Return Null
	Dim Input As InputStream
	Input.InitializeToBytesArray(0)
	Dim Bitmap As Bitmap
	Bitmap.Initialize2(Input)
	Input.Close
	Return Bitmap
End Sub
#End If

'Closes the store.
Public Sub Close
	sql1.Close
End Sub

Private Sub CorrectTable (Table As String) As String
	Return Table.Replace(" ", "_")
End Sub

Private Sub InsertTimeModified
	Put (UNIVERSE_SETTINGS, TIME_MODIFIED, Now)
End Sub